@@ -207,7 +207,7 @@ parse_xspf_track (TotemPlParser *parser, char *base, xmlDocPtr doc,
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 	}
 
-	fullpath = totem_pl_resolve_url (base, (char *)url);
+	fullpath = totem_pl_parser_resolve_url (base, (char *)url);
 	totem_pl_parser_add_one_url (parser, fullpath, (char *)title);
 	retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 
