@@ -39,8 +39,6 @@ static GtkFileFilter *filter_supported = NULL;
 static GtkFileFilter *filter_audio = NULL;
 static GtkFileFilter *filter_video = NULL;
 
-static char *xdg_user_dir_lookup (const char *type);
-
 gboolean
 totem_playing_dvd (const char *uri)
 {
@@ -82,36 +80,96 @@ totem_dot_dir (void)
 char *
 totem_pictures_dir (void)
 {
-	return xdg_user_dir_lookup ("PICTURES");
+	const char *dir;
+
+	dir = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
+	if (dir == NULL)
+		return NULL;
+	return g_strdup (dir);
 }
 
-gboolean
-totem_is_media (const char *uri)
+static GnomeVFSVolume *
+totem_get_volume_for_uri (GnomeVFSVolumeMonitor *monitor, const char *path)
 {
-	if (uri == NULL)
-		return FALSE;
+	GnomeVFSVolume *vol;
+	GnomeVFSDeviceType type;
 
-	if (g_str_has_prefix (uri, "cdda:") != FALSE)
-		return TRUE;
-	if (g_str_has_prefix (uri, "dvd:") != FALSE)
-		return TRUE;
-	if (g_str_has_prefix (uri, "vcd:") != FALSE)
-		return TRUE;
-	if (g_str_has_prefix (uri, "cd:") != FALSE)
-		return TRUE;
+	vol = gnome_vfs_volume_monitor_get_volume_for_path (monitor, path);
+	if (vol == NULL)
+		return NULL;
 
-	return FALSE;
+	type = gnome_vfs_volume_get_device_type (vol);
+	if (type != GNOME_VFS_DEVICE_TYPE_AUDIO_CD
+	    && type != GNOME_VFS_DEVICE_TYPE_VIDEO_DVD
+	    && type != GNOME_VFS_DEVICE_TYPE_CDROM) {
+	    	gnome_vfs_volume_unref (vol);
+	    	vol = NULL;
+	}
+
+	return vol;
 }
 
-gboolean
+static char *
+totem_get_mountpoint_for_dvd (const char *uri)
+{
+	if (g_str_has_prefix (uri, "dvd://") == FALSE)
+		return NULL;
+	return g_strdup (uri + strlen ("dvd://"));
+}
+
+static char *
+totem_get_mountpoint_for_vcd (const char *uri)
+{
+	return NULL;
+}
+
+GnomeVFSVolume *
+totem_get_volume_for_media (const char *uri)
+{
+	GnomeVFSVolumeMonitor *monitor;
+	GnomeVFSVolume *ret;
+	char *mount;
+
+	if (uri == NULL)
+		return NULL;
+
+	mount = NULL;
+	ret = NULL;
+
+	if (g_str_has_prefix (uri, "dvd://") != FALSE)
+		mount = totem_get_mountpoint_for_dvd (uri);
+	else if (g_str_has_prefix (uri, "vcd:") != FALSE)
+		mount = totem_get_mountpoint_for_vcd (uri);
+	else if (g_str_has_prefix (uri, "file:") != FALSE)
+		mount = g_filename_from_uri (uri, NULL, NULL);
+
+	if (mount == NULL)
+		return NULL;
+
+	monitor = gnome_vfs_get_volume_monitor ();
+	ret = totem_get_volume_for_uri (monitor, mount);
+	g_free (mount);
+
+	return ret;
+}
+
+static gboolean
 totem_is_special_mrl (const char *uri)
 {
-	if (uri == NULL)
+	GnomeVFSVolume *vol;
+	gboolean retval;
+
+	if (uri == NULL || g_str_has_prefix (uri, "file:") != FALSE)
 		return FALSE;
 	if (g_str_has_prefix (uri, "dvb:") != FALSE)
 		return TRUE;
 
-	return totem_is_media (uri);
+	vol = totem_get_volume_for_media (uri);
+	retval = (vol != NULL);
+	if (vol != NULL)
+		gnome_vfs_volume_unref (vol);
+
+	return retval;
 }
 
 gboolean
@@ -175,7 +233,8 @@ totem_create_full_path (const char *path)
 
 static void
 totem_action_on_unmount (GnomeVFSVolumeMonitor *vfsvolumemonitor,
-		GnomeVFSVolume *volume, Totem *totem)
+			 GnomeVFSVolume *volume,
+			 Totem *totem)
 {
 	totem_playlist_clear_with_gnome_vfs_volume (totem->playlist, volume);
 }
@@ -186,9 +245,9 @@ totem_setup_file_monitoring (Totem *totem)
 	totem->monitor = gnome_vfs_get_volume_monitor ();
 
 	g_signal_connect (G_OBJECT (totem->monitor),
-			"volume_pre_unmount",
-			G_CALLBACK (totem_action_on_unmount),
-			totem);
+			  "volume_pre_unmount",
+			  G_CALLBACK (totem_action_on_unmount),
+			  totem);
 }
 
 /* List from xine-lib's demux_sputext.c */
@@ -201,31 +260,16 @@ static const char subtitle_ext[][4] = {
 	"ssa"
 };
 
-char *
-totem_uri_get_subtitle_uri (const char *uri)
+static char *
+totem_uri_get_subtitle_for_uri (const char *uri)
 {
+	GnomeVFSURI *vfsuri;
 	char *subtitle;
 	guint len, i, suffix;
-	GnomeVFSURI *vfsuri;
-
-	if (g_str_has_prefix (uri, "http") != FALSE) {
-		return NULL;
-	}
-
-	/* Does gnome-vfs support that scheme? */
-	vfsuri = gnome_vfs_uri_new (uri);
-	if (vfsuri == NULL)
-		return NULL;
-	gnome_vfs_uri_unref (vfsuri);
-
-	/* Has the user specified a subtitle file manually? */
-	if (strstr (uri, "#subtitle:") != NULL) {
-		return NULL;
-	}
 
         /* Find the filename suffix delimiter */
 	len = strlen (uri);
-	for (suffix = len-1; suffix > 0; suffix--) {
+	for (suffix = len - 1; suffix > 0; suffix--) {
 		if (uri[suffix] == G_DIR_SEPARATOR ||
                     (uri[suffix] == '/')) {
 			/* This filename has no extension, we'll need to 
@@ -244,7 +288,10 @@ totem_uri_get_subtitle_uri (const char *uri)
 
 	/* Generate a subtitle string with room at the end to store the
 	 * 3 character extensions we want to search for */
-	subtitle = g_strdup_printf ("%.*s.???", suffix, uri);
+	subtitle = g_malloc0 (suffix + 4 + 1);
+	g_return_val_if_fail (subtitle != NULL, NULL);
+	g_strlcpy (subtitle, uri, suffix + 4 + 1);
+	g_strlcpy (subtitle + suffix, ".???", 5);
 	
 	/* Search for any files with one of our known subtitle extensions */
 	for (i = 0; i < G_N_ELEMENTS(subtitle_ext) ; i++) {
@@ -263,6 +310,62 @@ totem_uri_get_subtitle_uri (const char *uri)
 	return NULL;
 }
 
+static char *
+totem_uri_get_subtitle_in_subdir (GnomeVFSURI *vfsuri, const char *subdir)
+{
+	char *filename, *subtitle, *fullpath_str;
+	GnomeVFSURI *parent, *fullpath, *directory;
+
+	parent = gnome_vfs_uri_get_parent (vfsuri);
+	directory = gnome_vfs_uri_append_path (parent, subdir);
+	gnome_vfs_uri_unref (parent);
+
+	filename = g_path_get_basename (gnome_vfs_uri_get_path (vfsuri));
+	fullpath = gnome_vfs_uri_append_string (directory, filename);
+	gnome_vfs_uri_unref (directory);
+	g_free (filename);
+
+	fullpath_str = gnome_vfs_uri_to_string (fullpath, 0);
+	gnome_vfs_uri_unref (fullpath);
+	subtitle = totem_uri_get_subtitle_for_uri (fullpath_str);
+	g_free (fullpath_str);
+
+	return subtitle;
+}
+
+char *
+totem_uri_get_subtitle_uri (const char *uri)
+{
+	GnomeVFSURI *vfsuri;
+	char *subtitle;
+
+	if (g_str_has_prefix (uri, "http") != FALSE) {
+		return NULL;
+	}
+
+	/* Has the user specified a subtitle file manually? */
+	if (strstr (uri, "#subtitle:") != NULL) {
+		return NULL;
+	}
+
+	/* Does gnome-vfs support that scheme? */
+	vfsuri = gnome_vfs_uri_new (uri);
+	if (vfsuri == NULL)
+		return NULL;
+
+	/* Try in the current directory */
+	subtitle = totem_uri_get_subtitle_for_uri (uri);
+	if (subtitle != NULL) {
+		gnome_vfs_uri_unref (vfsuri);
+		return subtitle;
+	}
+
+	subtitle = totem_uri_get_subtitle_in_subdir (vfsuri, "subtitles");
+	gnome_vfs_uri_unref (vfsuri);
+
+	return subtitle;
+}
+
 char*
 totem_uri_escape_for_display (const char *uri)
 {
@@ -352,9 +455,9 @@ totem_destroy_file_filters (void)
 	}
 }
 
-static const char *dir_types[] = {
-	"VIDEOS",
-	"MUSIC"
+static const GUserDirectory dir_types[] = {
+	G_USER_DIRECTORY_VIDEOS,
+	G_USER_DIRECTORY_MUSIC
 };
 
 static void
@@ -362,28 +465,26 @@ totem_add_default_dirs (GtkFileChooser *dialog)
 {
 	guint i;
 	for (i = 0; i < G_N_ELEMENTS (dir_types); i++) {
-		char *dir;
+		const char *dir;
 
-		dir = xdg_user_dir_lookup (dir_types[i]);
+		dir = g_get_user_special_dir (dir_types[i]);
 		if (dir == NULL)
 			continue;
 		gtk_file_chooser_add_shortcut_folder (dialog, dir, NULL);
-		g_free (dir);
 	}
 }
 
 void
 totem_add_pictures_dir (GtkWidget *chooser)
 {
-	char *dir;
+	const char *dir;
 
 	g_return_if_fail (GTK_IS_FILE_CHOOSER (chooser) != FALSE);
 
-	dir = xdg_user_dir_lookup ("PICTURES");
+	dir = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
 	if (dir == NULL)
 		return;
 	gtk_file_chooser_add_shortcut_folder (GTK_FILE_CHOOSER (chooser), dir, NULL);
-	g_free (dir);
 }
 
 GSList *
@@ -394,6 +495,7 @@ totem_add_files (GtkWindow *parent, const char *path)
 	GSList *filenames;
 	char *mrl, *new_path;
 	GConfClient *conf;
+	gboolean set_folder;
 
 	fs = gtk_file_chooser_dialog_new (_("Select Movies or Playlists"),
 			parent,
@@ -411,18 +513,24 @@ totem_add_files (GtkWindow *parent, const char *path)
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (fs), FALSE);
 
 	conf = gconf_client_get_default ();
+	set_folder = TRUE;
 	if (path != NULL) {
-		gtk_file_chooser_set_current_folder_uri
+		set_folder = gtk_file_chooser_set_current_folder_uri
 			(GTK_FILE_CHOOSER (fs), path);
 	} else {
 		new_path = gconf_client_get_string (conf, "/apps/totem/open_path", NULL);
 		if (new_path != NULL && new_path != '\0') {
-			gtk_file_chooser_set_current_folder_uri
+			set_folder = gtk_file_chooser_set_current_folder_uri
 				(GTK_FILE_CHOOSER (fs), new_path);
 		}
 		g_free (new_path);
 	}
 
+	/* We didn't manage to change the directory */
+	if (set_folder == FALSE) {
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (fs),
+						     g_get_home_dir ());
+	}
 	totem_add_default_dirs (GTK_FILE_CHOOSER (fs));
 
 	response = gtk_dialog_run (GTK_DIALOG (fs));
@@ -457,124 +565,3 @@ totem_add_files (GtkWindow *parent, const char *path)
 	return filenames;
 }
 
-/* Copied from xdg-user-dir-lookup.c */
-#include <stdlib.h>
-
-static char *
-xdg_user_dir_lookup (const char *type)
-{
-  FILE *file;
-  char *home_dir, *config_home, *config_file;
-  char buffer[512];
-  char *user_dir;
-  char *p, *d;
-  int len;
-  int relative;
-  
-  home_dir = getenv ("HOME");
-
-  if (home_dir == NULL)
-    return strdup ("/tmp");
-
-  config_home = getenv ("XDG_CONFIG_HOME");
-  if (config_home == NULL || config_home[0] == 0)
-    {
-      config_file = malloc (strlen (home_dir) + strlen ("/.config/user-dirs.dirs") + 1);
-      strcpy (config_file, home_dir);
-      strcat (config_file, "/.config/user-dirs.dirs");
-    }
-  else
-    {
-      config_file = malloc (strlen (config_home) + strlen ("/user-dirs.dirs") + 1);
-      strcpy (config_file, config_home);
-      strcat (config_file, "/user-dirs.dirs");
-    }
-
-  file = fopen (config_file, "r");
-  free (config_file);
-  if (file == NULL)
-    goto error;
-
-  user_dir = NULL;
-  while (fgets (buffer, sizeof (buffer), file))
-    {
-      /* Remove newline at end */
-      len = strlen (buffer);
-      if (len > 0 && buffer[len-1] == '\n')
-	buffer[len-1] = 0;
-      
-      p = buffer;
-      while (*p == ' ' || *p == '\t')
-	p++;
-      
-      if (strncmp (p, "XDG_", 4) != 0)
-	continue;
-      p += 4;
-      if (strncmp (p, type, strlen (type)) != 0)
-	continue;
-      p += strlen (type);
-      if (strncmp (p, "_DIR", 4) != 0)
-	continue;
-      p += 4;
-
-      while (*p == ' ' || *p == '\t')
-	p++;
-
-      if (*p != '=')
-	continue;
-      p++;
-      
-      while (*p == ' ' || *p == '\t')
-	p++;
-
-      if (*p != '"')
-	continue;
-      p++;
-      
-      relative = 0;
-      if (strncmp (p, "$HOME/", 6) == 0)
-	{
-	  p += 6;
-	  relative = 1;
-	}
-      else if (*p != '/')
-	continue;
-      
-      if (relative)
-	{
-	  user_dir = malloc (strlen (home_dir) + 1 + strlen (p) + 1);
-	  strcpy (user_dir, home_dir);
-	  strcat (user_dir, "/");
-	}
-      else
-	{
-	  user_dir = malloc (strlen (p) + 1);
-	  *user_dir = 0;
-	}
-      
-      d = user_dir + strlen (user_dir);
-      while (*p && *p != '"')
-	{
-	  if ((*p == '\\') && (*(p+1) != 0))
-	    p++;
-	  *d++ = *p++;
-	}
-      *d = 0;
-    }  
-  fclose (file);
-
-  if (user_dir)
-    return user_dir;
-
- error:
-  /* Special case desktop for historical compatibility */
-  if (strcmp (type, "DESKTOP") == 0)
-    {
-      user_dir = malloc (strlen (home_dir) + strlen ("/Desktop") + 1);
-      strcpy (user_dir, home_dir);
-      strcat (user_dir, "/Desktop");
-      return user_dir;
-    }
-  else
-    return strdup (home_dir);
-}
